/****************************************************************************
** Meta object code from reading C++ file 'camerabinsession.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../git/src/plugins/gstreamer/camerabin/camerabinsession.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'camerabinsession.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CameraBinSession_t {
    QByteArrayData data[30];
    char stringdata0[383];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CameraBinSession_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CameraBinSession_t qt_meta_stringdata_CameraBinSession = {
    {
QT_MOC_LITERAL(0, 0, 16), // "CameraBinSession"
QT_MOC_LITERAL(1, 17, 13), // "statusChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 15), // "QCamera::Status"
QT_MOC_LITERAL(4, 48, 6), // "status"
QT_MOC_LITERAL(5, 55, 19), // "pendingStateChanged"
QT_MOC_LITERAL(6, 75, 14), // "QCamera::State"
QT_MOC_LITERAL(7, 90, 5), // "state"
QT_MOC_LITERAL(8, 96, 15), // "durationChanged"
QT_MOC_LITERAL(9, 112, 8), // "duration"
QT_MOC_LITERAL(10, 121, 5), // "error"
QT_MOC_LITERAL(11, 127, 11), // "errorString"
QT_MOC_LITERAL(12, 139, 12), // "imageExposed"
QT_MOC_LITERAL(13, 152, 9), // "requestId"
QT_MOC_LITERAL(14, 162, 13), // "imageCaptured"
QT_MOC_LITERAL(15, 176, 3), // "img"
QT_MOC_LITERAL(16, 180, 12), // "mutedChanged"
QT_MOC_LITERAL(17, 193, 17), // "viewfinderChanged"
QT_MOC_LITERAL(18, 211, 12), // "readyChanged"
QT_MOC_LITERAL(19, 224, 11), // "busyChanged"
QT_MOC_LITERAL(20, 236, 9), // "setDevice"
QT_MOC_LITERAL(21, 246, 6), // "device"
QT_MOC_LITERAL(22, 253, 8), // "setState"
QT_MOC_LITERAL(23, 262, 16), // "setCaptureDevice"
QT_MOC_LITERAL(24, 279, 10), // "deviceName"
QT_MOC_LITERAL(25, 290, 11), // "setMetaData"
QT_MOC_LITERAL(26, 302, 25), // "QMap<QByteArray,QVariant>"
QT_MOC_LITERAL(27, 328, 8), // "setMuted"
QT_MOC_LITERAL(28, 337, 22), // "handleViewfinderChange"
QT_MOC_LITERAL(29, 360, 22) // "setupCaptureResolution"

    },
    "CameraBinSession\0statusChanged\0\0"
    "QCamera::Status\0status\0pendingStateChanged\0"
    "QCamera::State\0state\0durationChanged\0"
    "duration\0error\0errorString\0imageExposed\0"
    "requestId\0imageCaptured\0img\0mutedChanged\0"
    "viewfinderChanged\0readyChanged\0"
    "busyChanged\0setDevice\0device\0setState\0"
    "setCaptureDevice\0deviceName\0setMetaData\0"
    "QMap<QByteArray,QVariant>\0setMuted\0"
    "handleViewfinderChange\0setupCaptureResolution"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CameraBinSession[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       1,  148, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x06 /* Public */,
       5,    1,  102,    2, 0x06 /* Public */,
       8,    1,  105,    2, 0x06 /* Public */,
      10,    2,  108,    2, 0x06 /* Public */,
      12,    1,  113,    2, 0x06 /* Public */,
      14,    2,  116,    2, 0x06 /* Public */,
      16,    1,  121,    2, 0x06 /* Public */,
      17,    0,  124,    2, 0x06 /* Public */,
      18,    1,  125,    2, 0x06 /* Public */,
      19,    1,  128,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      20,    1,  131,    2, 0x0a /* Public */,
      22,    1,  134,    2, 0x0a /* Public */,
      23,    1,  137,    2, 0x0a /* Public */,
      25,    1,  140,    2, 0x0a /* Public */,
      27,    1,  143,    2, 0x0a /* Public */,
      28,    0,  146,    2, 0x08 /* Private */,
      29,    0,  147,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::LongLong,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   10,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Int, QMetaType::QImage,   13,   15,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   21,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, 0x80000000 | 26,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       9, QMetaType::LongLong, 0x00495001,

 // properties: notify_signal_id
       2,

       0        // eod
};

void CameraBinSession::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CameraBinSession *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->statusChanged((*reinterpret_cast< QCamera::Status(*)>(_a[1]))); break;
        case 1: _t->pendingStateChanged((*reinterpret_cast< QCamera::State(*)>(_a[1]))); break;
        case 2: _t->durationChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 3: _t->error((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->imageExposed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->imageCaptured((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QImage(*)>(_a[2]))); break;
        case 6: _t->mutedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->viewfinderChanged(); break;
        case 8: _t->readyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->busyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->setDevice((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->setState((*reinterpret_cast< QCamera::State(*)>(_a[1]))); break;
        case 12: _t->setCaptureDevice((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->setMetaData((*reinterpret_cast< const QMap<QByteArray,QVariant>(*)>(_a[1]))); break;
        case 14: _t->setMuted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->handleViewfinderChange(); break;
        case 16: _t->setupCaptureResolution(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::Status >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::State >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::State >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CameraBinSession::*)(QCamera::Status );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::statusChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(QCamera::State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::pendingStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::durationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::error)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::imageExposed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(int , const QImage & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::imageCaptured)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::mutedChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::viewfinderChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::readyChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (CameraBinSession::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraBinSession::busyChanged)) {
                *result = 9;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<CameraBinSession *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qint64*>(_v) = _t->duration(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject CameraBinSession::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CameraBinSession.data,
    qt_meta_data_CameraBinSession,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CameraBinSession::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CameraBinSession::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CameraBinSession.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGstreamerBusMessageFilter"))
        return static_cast< QGstreamerBusMessageFilter*>(this);
    if (!strcmp(_clname, "QGstreamerSyncMessageFilter"))
        return static_cast< QGstreamerSyncMessageFilter*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.gstreamerbusmessagefilter/5.0"))
        return static_cast< QGstreamerBusMessageFilter*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.gstreamersyncmessagefilter/5.0"))
        return static_cast< QGstreamerSyncMessageFilter*>(this);
    return QObject::qt_metacast(_clname);
}

int CameraBinSession::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void CameraBinSession::statusChanged(QCamera::Status _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CameraBinSession::pendingStateChanged(QCamera::State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CameraBinSession::durationChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void CameraBinSession::error(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void CameraBinSession::imageExposed(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void CameraBinSession::imageCaptured(int _t1, const QImage & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void CameraBinSession::mutedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void CameraBinSession::viewfinderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void CameraBinSession::readyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void CameraBinSession::busyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
