/****************************************************************************
** Meta object code from reading C++ file 'qgstreamerplayerserviceplugin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../git/src/plugins/gstreamer/mediaplayer/qgstreamerplayerserviceplugin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgstreamerplayerserviceplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QGstreamerPlayerServicePlugin_t {
    QByteArrayData data[1];
    char stringdata0[30];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QGstreamerPlayerServicePlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QGstreamerPlayerServicePlugin_t qt_meta_stringdata_QGstreamerPlayerServicePlugin = {
    {
QT_MOC_LITERAL(0, 0, 29) // "QGstreamerPlayerServicePlugin"

    },
    "QGstreamerPlayerServicePlugin"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QGstreamerPlayerServicePlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QGstreamerPlayerServicePlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QGstreamerPlayerServicePlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QMediaServiceProviderPlugin::staticMetaObject>(),
    qt_meta_stringdata_QGstreamerPlayerServicePlugin.data,
    qt_meta_data_QGstreamerPlayerServicePlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QGstreamerPlayerServicePlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGstreamerPlayerServicePlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QGstreamerPlayerServicePlugin.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QMediaServiceFeaturesInterface"))
        return static_cast< QMediaServiceFeaturesInterface*>(this);
    if (!strcmp(_clname, "QMediaServiceSupportedFormatsInterface"))
        return static_cast< QMediaServiceSupportedFormatsInterface*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.mediaservicefeatures/5.0"))
        return static_cast< QMediaServiceFeaturesInterface*>(this);
    if (!strcmp(_clname, "org.qt-project.qt.mediaservicesupportedformats/5.0"))
        return static_cast< QMediaServiceSupportedFormatsInterface*>(this);
    return QMediaServiceProviderPlugin::qt_metacast(_clname);
}

int QGstreamerPlayerServicePlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMediaServiceProviderPlugin::qt_metacall(_c, _id, _a);
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x31,  'o',  'r',  'g',  '.',  'q', 
    't',  '-',  'p',  'r',  'o',  'j',  'e',  'c', 
    't',  '.',  'q',  't',  '.',  'm',  'e',  'd', 
    'i',  'a',  's',  'e',  'r',  'v',  'i',  'c', 
    'e',  'p',  'r',  'o',  'v',  'i',  'd',  'e', 
    'r',  'f',  'a',  'c',  't',  'o',  'r',  'y', 
    '/',  '5',  '.',  '0', 
    // "className"
    0x03,  0x78,  0x1d,  'Q',  'G',  's',  't',  'r', 
    'e',  'a',  'm',  'e',  'r',  'P',  'l',  'a', 
    'y',  'e',  'r',  'S',  'e',  'r',  'v',  'i', 
    'c',  'e',  'P',  'l',  'u',  'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xa2,  0x64,  'K',  'e',  'y',  's',  0x81, 
    0x74,  'g',  's',  't',  'r',  'e',  'a',  'm', 
    'e',  'r',  'm',  'e',  'd',  'i',  'a',  'p', 
    'l',  'a',  'y',  'e',  'r',  0x68,  'S',  'e', 
    'r',  'v',  'i',  'c',  'e',  's',  0x81,  0x78, 
    0x1d,  'o',  'r',  'g',  '.',  'q',  't',  '-', 
    'p',  'r',  'o',  'j',  'e',  'c',  't',  '.', 
    'q',  't',  '.',  'm',  'e',  'd',  'i',  'a', 
    'p',  'l',  'a',  'y',  'e',  'r', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(QGstreamerPlayerServicePlugin, QGstreamerPlayerServicePlugin)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
