/****************************************************************************
** Meta object code from reading C++ file 'qcamera.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/multimedia/camera/qcamera.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qcamera.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QCamera_t {
    QByteArrayData data[81];
    char stringdata0[1023];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QCamera_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QCamera_t qt_meta_stringdata_QCamera = {
    {
QT_MOC_LITERAL(0, 0, 7), // "QCamera"
QT_MOC_LITERAL(1, 8, 12), // "stateChanged"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 14), // "QCamera::State"
QT_MOC_LITERAL(4, 37, 5), // "state"
QT_MOC_LITERAL(5, 43, 18), // "captureModeChanged"
QT_MOC_LITERAL(6, 62, 21), // "QCamera::CaptureModes"
QT_MOC_LITERAL(7, 84, 13), // "statusChanged"
QT_MOC_LITERAL(8, 98, 15), // "QCamera::Status"
QT_MOC_LITERAL(9, 114, 6), // "status"
QT_MOC_LITERAL(10, 121, 6), // "locked"
QT_MOC_LITERAL(11, 128, 10), // "lockFailed"
QT_MOC_LITERAL(12, 139, 17), // "lockStatusChanged"
QT_MOC_LITERAL(13, 157, 19), // "QCamera::LockStatus"
QT_MOC_LITERAL(14, 177, 25), // "QCamera::LockChangeReason"
QT_MOC_LITERAL(15, 203, 6), // "reason"
QT_MOC_LITERAL(16, 210, 17), // "QCamera::LockType"
QT_MOC_LITERAL(17, 228, 4), // "lock"
QT_MOC_LITERAL(18, 233, 5), // "error"
QT_MOC_LITERAL(19, 239, 14), // "QCamera::Error"
QT_MOC_LITERAL(20, 254, 13), // "errorOccurred"
QT_MOC_LITERAL(21, 268, 14), // "setCaptureMode"
QT_MOC_LITERAL(22, 283, 4), // "mode"
QT_MOC_LITERAL(23, 288, 4), // "load"
QT_MOC_LITERAL(24, 293, 6), // "unload"
QT_MOC_LITERAL(25, 300, 5), // "start"
QT_MOC_LITERAL(26, 306, 4), // "stop"
QT_MOC_LITERAL(27, 311, 13), // "searchAndLock"
QT_MOC_LITERAL(28, 325, 6), // "unlock"
QT_MOC_LITERAL(29, 332, 18), // "QCamera::LockTypes"
QT_MOC_LITERAL(30, 351, 5), // "locks"
QT_MOC_LITERAL(31, 357, 24), // "_q_preparePropertyChange"
QT_MOC_LITERAL(32, 382, 16), // "_q_restartCamera"
QT_MOC_LITERAL(33, 399, 8), // "_q_error"
QT_MOC_LITERAL(34, 408, 19), // "_q_updateLockStatus"
QT_MOC_LITERAL(35, 428, 14), // "_q_updateState"
QT_MOC_LITERAL(36, 443, 11), // "captureMode"
QT_MOC_LITERAL(37, 455, 10), // "lockStatus"
QT_MOC_LITERAL(38, 466, 6), // "Status"
QT_MOC_LITERAL(39, 473, 17), // "UnavailableStatus"
QT_MOC_LITERAL(40, 491, 14), // "UnloadedStatus"
QT_MOC_LITERAL(41, 506, 13), // "LoadingStatus"
QT_MOC_LITERAL(42, 520, 15), // "UnloadingStatus"
QT_MOC_LITERAL(43, 536, 12), // "LoadedStatus"
QT_MOC_LITERAL(44, 549, 13), // "StandbyStatus"
QT_MOC_LITERAL(45, 563, 14), // "StartingStatus"
QT_MOC_LITERAL(46, 578, 14), // "StoppingStatus"
QT_MOC_LITERAL(47, 593, 12), // "ActiveStatus"
QT_MOC_LITERAL(48, 606, 5), // "State"
QT_MOC_LITERAL(49, 612, 13), // "UnloadedState"
QT_MOC_LITERAL(50, 626, 11), // "LoadedState"
QT_MOC_LITERAL(51, 638, 11), // "ActiveState"
QT_MOC_LITERAL(52, 650, 11), // "CaptureMode"
QT_MOC_LITERAL(53, 662, 17), // "CaptureViewfinder"
QT_MOC_LITERAL(54, 680, 17), // "CaptureStillImage"
QT_MOC_LITERAL(55, 698, 12), // "CaptureVideo"
QT_MOC_LITERAL(56, 711, 5), // "Error"
QT_MOC_LITERAL(57, 717, 7), // "NoError"
QT_MOC_LITERAL(58, 725, 11), // "CameraError"
QT_MOC_LITERAL(59, 737, 19), // "InvalidRequestError"
QT_MOC_LITERAL(60, 757, 19), // "ServiceMissingError"
QT_MOC_LITERAL(61, 777, 24), // "NotSupportedFeatureError"
QT_MOC_LITERAL(62, 802, 10), // "LockStatus"
QT_MOC_LITERAL(63, 813, 8), // "Unlocked"
QT_MOC_LITERAL(64, 822, 9), // "Searching"
QT_MOC_LITERAL(65, 832, 6), // "Locked"
QT_MOC_LITERAL(66, 839, 16), // "LockChangeReason"
QT_MOC_LITERAL(67, 856, 11), // "UserRequest"
QT_MOC_LITERAL(68, 868, 12), // "LockAcquired"
QT_MOC_LITERAL(69, 881, 10), // "LockFailed"
QT_MOC_LITERAL(70, 892, 8), // "LockLost"
QT_MOC_LITERAL(71, 901, 17), // "LockTemporaryLost"
QT_MOC_LITERAL(72, 919, 8), // "LockType"
QT_MOC_LITERAL(73, 928, 6), // "NoLock"
QT_MOC_LITERAL(74, 935, 12), // "LockExposure"
QT_MOC_LITERAL(75, 948, 16), // "LockWhiteBalance"
QT_MOC_LITERAL(76, 965, 9), // "LockFocus"
QT_MOC_LITERAL(77, 975, 8), // "Position"
QT_MOC_LITERAL(78, 984, 19), // "UnspecifiedPosition"
QT_MOC_LITERAL(79, 1004, 8), // "BackFace"
QT_MOC_LITERAL(80, 1013, 9) // "FrontFace"

    },
    "QCamera\0stateChanged\0\0QCamera::State\0"
    "state\0captureModeChanged\0QCamera::CaptureModes\0"
    "statusChanged\0QCamera::Status\0status\0"
    "locked\0lockFailed\0lockStatusChanged\0"
    "QCamera::LockStatus\0QCamera::LockChangeReason\0"
    "reason\0QCamera::LockType\0lock\0error\0"
    "QCamera::Error\0errorOccurred\0"
    "setCaptureMode\0mode\0load\0unload\0start\0"
    "stop\0searchAndLock\0unlock\0QCamera::LockTypes\0"
    "locks\0_q_preparePropertyChange\0"
    "_q_restartCamera\0_q_error\0_q_updateLockStatus\0"
    "_q_updateState\0captureMode\0lockStatus\0"
    "Status\0UnavailableStatus\0UnloadedStatus\0"
    "LoadingStatus\0UnloadingStatus\0"
    "LoadedStatus\0StandbyStatus\0StartingStatus\0"
    "StoppingStatus\0ActiveStatus\0State\0"
    "UnloadedState\0LoadedState\0ActiveState\0"
    "CaptureMode\0CaptureViewfinder\0"
    "CaptureStillImage\0CaptureVideo\0Error\0"
    "NoError\0CameraError\0InvalidRequestError\0"
    "ServiceMissingError\0NotSupportedFeatureError\0"
    "LockStatus\0Unlocked\0Searching\0Locked\0"
    "LockChangeReason\0UserRequest\0LockAcquired\0"
    "LockFailed\0LockLost\0LockTemporaryLost\0"
    "LockType\0NoLock\0LockExposure\0"
    "LockWhiteBalance\0LockFocus\0Position\0"
    "UnspecifiedPosition\0BackFace\0FrontFace"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QCamera[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       4,  192, // properties
       8,  208, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  129,    2, 0x06 /* Public */,
       5,    1,  132,    2, 0x06 /* Public */,
       7,    1,  135,    2, 0x06 /* Public */,
      10,    0,  138,    2, 0x06 /* Public */,
      11,    0,  139,    2, 0x06 /* Public */,
      12,    2,  140,    2, 0x06 /* Public */,
      12,    3,  145,    2, 0x06 /* Public */,
      18,    1,  152,    2, 0x06 /* Public */,
      20,    1,  155,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      21,    1,  158,    2, 0x0a /* Public */,
      23,    0,  161,    2, 0x0a /* Public */,
      24,    0,  162,    2, 0x0a /* Public */,
      25,    0,  163,    2, 0x0a /* Public */,
      26,    0,  164,    2, 0x0a /* Public */,
      27,    0,  165,    2, 0x0a /* Public */,
      28,    0,  166,    2, 0x0a /* Public */,
      27,    1,  167,    2, 0x0a /* Public */,
      28,    1,  170,    2, 0x0a /* Public */,
      31,    1,  173,    2, 0x08 /* Private */,
      32,    0,  176,    2, 0x08 /* Private */,
      33,    2,  177,    2, 0x08 /* Private */,
      34,    3,  182,    2, 0x08 /* Private */,
      35,    1,  189,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 14,    9,   15,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 13, 0x80000000 | 14,   17,    9,   15,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 19,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 6,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 29,   30,
    QMetaType::Void, 0x80000000 | 29,   30,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    2,    2,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 13, 0x80000000 | 14,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,

 // properties: name, type, flags
       4, 0x80000000 | 3, 0x00495009,
       9, 0x80000000 | 8, 0x00495009,
      36, 0x80000000 | 6, 0x0049510b,
      37, 0x80000000 | 13, 0x00495009,

 // properties: notify_signal_id
       0,
       2,
       1,
       5,

 // enums: name, alias, flags, count, data
      38,   38, 0x0,    9,  248,
      48,   48, 0x0,    3,  266,
      52,   52, 0x0,    3,  272,
      56,   56, 0x0,    5,  278,
      62,   62, 0x0,    3,  288,
      66,   66, 0x0,    5,  294,
      72,   72, 0x0,    4,  304,
      77,   77, 0x0,    3,  312,

 // enum data: key, value
      39, uint(QCamera::UnavailableStatus),
      40, uint(QCamera::UnloadedStatus),
      41, uint(QCamera::LoadingStatus),
      42, uint(QCamera::UnloadingStatus),
      43, uint(QCamera::LoadedStatus),
      44, uint(QCamera::StandbyStatus),
      45, uint(QCamera::StartingStatus),
      46, uint(QCamera::StoppingStatus),
      47, uint(QCamera::ActiveStatus),
      49, uint(QCamera::UnloadedState),
      50, uint(QCamera::LoadedState),
      51, uint(QCamera::ActiveState),
      53, uint(QCamera::CaptureViewfinder),
      54, uint(QCamera::CaptureStillImage),
      55, uint(QCamera::CaptureVideo),
      57, uint(QCamera::NoError),
      58, uint(QCamera::CameraError),
      59, uint(QCamera::InvalidRequestError),
      60, uint(QCamera::ServiceMissingError),
      61, uint(QCamera::NotSupportedFeatureError),
      63, uint(QCamera::Unlocked),
      64, uint(QCamera::Searching),
      65, uint(QCamera::Locked),
      67, uint(QCamera::UserRequest),
      68, uint(QCamera::LockAcquired),
      69, uint(QCamera::LockFailed),
      70, uint(QCamera::LockLost),
      71, uint(QCamera::LockTemporaryLost),
      73, uint(QCamera::NoLock),
      74, uint(QCamera::LockExposure),
      75, uint(QCamera::LockWhiteBalance),
      76, uint(QCamera::LockFocus),
      78, uint(QCamera::UnspecifiedPosition),
      79, uint(QCamera::BackFace),
      80, uint(QCamera::FrontFace),

       0        // eod
};

void QCamera::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QCamera *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast< QCamera::State(*)>(_a[1]))); break;
        case 1: _t->captureModeChanged((*reinterpret_cast< QCamera::CaptureModes(*)>(_a[1]))); break;
        case 2: _t->statusChanged((*reinterpret_cast< QCamera::Status(*)>(_a[1]))); break;
        case 3: _t->locked(); break;
        case 4: _t->lockFailed(); break;
        case 5: _t->lockStatusChanged((*reinterpret_cast< QCamera::LockStatus(*)>(_a[1])),(*reinterpret_cast< QCamera::LockChangeReason(*)>(_a[2]))); break;
        case 6: _t->lockStatusChanged((*reinterpret_cast< QCamera::LockType(*)>(_a[1])),(*reinterpret_cast< QCamera::LockStatus(*)>(_a[2])),(*reinterpret_cast< QCamera::LockChangeReason(*)>(_a[3]))); break;
        case 7: _t->error((*reinterpret_cast< QCamera::Error(*)>(_a[1]))); break;
        case 8: _t->errorOccurred((*reinterpret_cast< QCamera::Error(*)>(_a[1]))); break;
        case 9: _t->setCaptureMode((*reinterpret_cast< QCamera::CaptureModes(*)>(_a[1]))); break;
        case 10: _t->load(); break;
        case 11: _t->unload(); break;
        case 12: _t->start(); break;
        case 13: _t->stop(); break;
        case 14: _t->searchAndLock(); break;
        case 15: _t->unlock(); break;
        case 16: _t->searchAndLock((*reinterpret_cast< QCamera::LockTypes(*)>(_a[1]))); break;
        case 17: _t->unlock((*reinterpret_cast< QCamera::LockTypes(*)>(_a[1]))); break;
        case 18: _t->d_func()->_q_preparePropertyChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->d_func()->_q_restartCamera(); break;
        case 20: _t->d_func()->_q_error((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 21: _t->d_func()->_q_updateLockStatus((*reinterpret_cast< QCamera::LockType(*)>(_a[1])),(*reinterpret_cast< QCamera::LockStatus(*)>(_a[2])),(*reinterpret_cast< QCamera::LockChangeReason(*)>(_a[3]))); break;
        case 22: _t->d_func()->_q_updateState((*reinterpret_cast< QCamera::State(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::State >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::CaptureModes >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::Status >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockChangeReason >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockStatus >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockChangeReason >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockStatus >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockType >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::Error >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::Error >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::CaptureModes >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockChangeReason >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockStatus >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockType >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::State >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QCamera::*)(QCamera::State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::stateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QCamera::*)(QCamera::CaptureModes );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::captureModeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QCamera::*)(QCamera::Status );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::statusChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QCamera::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::locked)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QCamera::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::lockFailed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QCamera::*)(QCamera::LockStatus , QCamera::LockChangeReason );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::lockStatusChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QCamera::*)(QCamera::LockType , QCamera::LockStatus , QCamera::LockChangeReason );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::lockStatusChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QCamera::*)(QCamera::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::error)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QCamera::*)(QCamera::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QCamera::errorOccurred)) {
                *result = 8;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::CaptureModes >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::LockStatus >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::State >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QCamera::Status >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QCamera *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QCamera::State*>(_v) = _t->state(); break;
        case 1: *reinterpret_cast< QCamera::Status*>(_v) = _t->status(); break;
        case 2: *reinterpret_cast< QCamera::CaptureModes*>(_v) = _t->captureMode(); break;
        case 3: *reinterpret_cast< QCamera::LockStatus*>(_v) = _t->lockStatus(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QCamera *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setCaptureMode(*reinterpret_cast< QCamera::CaptureModes*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QCamera::staticMetaObject = { {
    QMetaObject::SuperData::link<QMediaObject::staticMetaObject>(),
    qt_meta_stringdata_QCamera.data,
    qt_meta_data_QCamera,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QCamera::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QCamera::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QCamera.stringdata0))
        return static_cast<void*>(this);
    return QMediaObject::qt_metacast(_clname);
}

int QCamera::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMediaObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QCamera::stateChanged(QCamera::State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QCamera::captureModeChanged(QCamera::CaptureModes _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QCamera::statusChanged(QCamera::Status _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QCamera::locked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QCamera::lockFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QCamera::lockStatusChanged(QCamera::LockStatus _t1, QCamera::LockChangeReason _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QCamera::lockStatusChanged(QCamera::LockType _t1, QCamera::LockStatus _t2, QCamera::LockChangeReason _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QCamera::error(QCamera::Error _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QCamera::errorOccurred(QCamera::Error _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
