/****************************************************************************
** Meta object code from reading C++ file 'qsoundeffect.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/multimedia/audio/qsoundeffect.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsoundeffect.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QSoundEffect_t {
    QByteArrayData data[30];
    char stringdata0[295];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QSoundEffect_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QSoundEffect_t qt_meta_stringdata_QSoundEffect = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QSoundEffect"
QT_MOC_LITERAL(1, 13, 13), // "DefaultMethod"
QT_MOC_LITERAL(2, 27, 6), // "play()"
QT_MOC_LITERAL(3, 34, 13), // "sourceChanged"
QT_MOC_LITERAL(4, 48, 0), // ""
QT_MOC_LITERAL(5, 49, 16), // "loopCountChanged"
QT_MOC_LITERAL(6, 66, 21), // "loopsRemainingChanged"
QT_MOC_LITERAL(7, 88, 13), // "volumeChanged"
QT_MOC_LITERAL(8, 102, 12), // "mutedChanged"
QT_MOC_LITERAL(9, 115, 13), // "loadedChanged"
QT_MOC_LITERAL(10, 129, 14), // "playingChanged"
QT_MOC_LITERAL(11, 144, 13), // "statusChanged"
QT_MOC_LITERAL(12, 158, 15), // "categoryChanged"
QT_MOC_LITERAL(13, 174, 4), // "play"
QT_MOC_LITERAL(14, 179, 4), // "stop"
QT_MOC_LITERAL(15, 184, 6), // "source"
QT_MOC_LITERAL(16, 191, 5), // "loops"
QT_MOC_LITERAL(17, 197, 14), // "loopsRemaining"
QT_MOC_LITERAL(18, 212, 6), // "volume"
QT_MOC_LITERAL(19, 219, 5), // "muted"
QT_MOC_LITERAL(20, 225, 7), // "playing"
QT_MOC_LITERAL(21, 233, 6), // "status"
QT_MOC_LITERAL(22, 240, 6), // "Status"
QT_MOC_LITERAL(23, 247, 8), // "category"
QT_MOC_LITERAL(24, 256, 4), // "Loop"
QT_MOC_LITERAL(25, 261, 8), // "Infinite"
QT_MOC_LITERAL(26, 270, 4), // "Null"
QT_MOC_LITERAL(27, 275, 7), // "Loading"
QT_MOC_LITERAL(28, 283, 5), // "Ready"
QT_MOC_LITERAL(29, 289, 5) // "Error"

    },
    "QSoundEffect\0DefaultMethod\0play()\0"
    "sourceChanged\0\0loopCountChanged\0"
    "loopsRemainingChanged\0volumeChanged\0"
    "mutedChanged\0loadedChanged\0playingChanged\0"
    "statusChanged\0categoryChanged\0play\0"
    "stop\0source\0loops\0loopsRemaining\0"
    "volume\0muted\0playing\0status\0Status\0"
    "category\0Loop\0Infinite\0Null\0Loading\0"
    "Ready\0Error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QSoundEffect[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      11,   16, // methods
       8,   82, // properties
       2,  114, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   71,    4, 0x06 /* Public */,
       5,    0,   72,    4, 0x06 /* Public */,
       6,    0,   73,    4, 0x06 /* Public */,
       7,    0,   74,    4, 0x06 /* Public */,
       8,    0,   75,    4, 0x06 /* Public */,
       9,    0,   76,    4, 0x06 /* Public */,
      10,    0,   77,    4, 0x06 /* Public */,
      11,    0,   78,    4, 0x06 /* Public */,
      12,    0,   79,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    0,   80,    4, 0x0a /* Public */,
      14,    0,   81,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      15, QMetaType::QUrl, 0x00495103,
      16, QMetaType::Int, 0x00495003,
      17, QMetaType::Int, 0x00495001,
      18, QMetaType::QReal, 0x00495103,
      19, QMetaType::Bool, 0x00495103,
      20, QMetaType::Bool, 0x00495001,
      21, 0x80000000 | 22, 0x00495009,
      23, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       6,
       7,
       8,

 // enums: name, alias, flags, count, data
      24,   24, 0x0,    1,  124,
      22,   22, 0x0,    4,  126,

 // enum data: key, value
      25, uint(QSoundEffect::Infinite),
      26, uint(QSoundEffect::Null),
      27, uint(QSoundEffect::Loading),
      28, uint(QSoundEffect::Ready),
      29, uint(QSoundEffect::Error),

       0        // eod
};

void QSoundEffect::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QSoundEffect *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sourceChanged(); break;
        case 1: _t->loopCountChanged(); break;
        case 2: _t->loopsRemainingChanged(); break;
        case 3: _t->volumeChanged(); break;
        case 4: _t->mutedChanged(); break;
        case 5: _t->loadedChanged(); break;
        case 6: _t->playingChanged(); break;
        case 7: _t->statusChanged(); break;
        case 8: _t->categoryChanged(); break;
        case 9: _t->play(); break;
        case 10: _t->stop(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::sourceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::loopCountChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::loopsRemainingChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::volumeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::mutedChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::loadedChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::playingChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::statusChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QSoundEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSoundEffect::categoryChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QSoundEffect *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = _t->source(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->loopCount(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->loopsRemaining(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->volume(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isMuted(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->isPlaying(); break;
        case 6: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->category(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QSoundEffect *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSource(*reinterpret_cast< QUrl*>(_v)); break;
        case 1: _t->setLoopCount(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setVolume(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setMuted(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setCategory(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QSoundEffect::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QSoundEffect.data,
    qt_meta_data_QSoundEffect,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QSoundEffect::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSoundEffect::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QSoundEffect.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QSoundEffect::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QSoundEffect::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QSoundEffect::loopCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QSoundEffect::loopsRemainingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QSoundEffect::volumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QSoundEffect::mutedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QSoundEffect::loadedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QSoundEffect::playingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QSoundEffect::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QSoundEffect::categoryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
