/****************************************************************************
** Meta object code from reading C++ file 'qgstreamerplayercontrol_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.13)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/gsttools/qgstreamerplayercontrol_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgstreamerplayercontrol_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.13. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QGstreamerPlayerControl_t {
    QByteArrayData data[22];
    char stringdata0[274];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QGstreamerPlayerControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QGstreamerPlayerControl_t qt_meta_stringdata_QGstreamerPlayerControl = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QGstreamerPlayerControl"
QT_MOC_LITERAL(1, 24, 11), // "setPosition"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 3), // "pos"
QT_MOC_LITERAL(4, 41, 4), // "play"
QT_MOC_LITERAL(5, 46, 5), // "pause"
QT_MOC_LITERAL(6, 52, 4), // "stop"
QT_MOC_LITERAL(7, 57, 9), // "setVolume"
QT_MOC_LITERAL(8, 67, 6), // "volume"
QT_MOC_LITERAL(9, 74, 8), // "setMuted"
QT_MOC_LITERAL(10, 83, 5), // "muted"
QT_MOC_LITERAL(11, 89, 18), // "updateSessionState"
QT_MOC_LITERAL(12, 108, 19), // "QMediaPlayer::State"
QT_MOC_LITERAL(13, 128, 5), // "state"
QT_MOC_LITERAL(14, 134, 17), // "updateMediaStatus"
QT_MOC_LITERAL(15, 152, 10), // "processEOS"
QT_MOC_LITERAL(16, 163, 17), // "setBufferProgress"
QT_MOC_LITERAL(17, 181, 8), // "progress"
QT_MOC_LITERAL(18, 190, 18), // "handleInvalidMedia"
QT_MOC_LITERAL(19, 209, 22), // "handleResourcesGranted"
QT_MOC_LITERAL(20, 232, 19), // "handleResourcesLost"
QT_MOC_LITERAL(21, 252, 21) // "handleResourcesDenied"

    },
    "QGstreamerPlayerControl\0setPosition\0"
    "\0pos\0play\0pause\0stop\0setVolume\0volume\0"
    "setMuted\0muted\0updateSessionState\0"
    "QMediaPlayer::State\0state\0updateMediaStatus\0"
    "processEOS\0setBufferProgress\0progress\0"
    "handleInvalidMedia\0handleResourcesGranted\0"
    "handleResourcesLost\0handleResourcesDenied"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QGstreamerPlayerControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x0a /* Public */,
       4,    0,   87,    2, 0x0a /* Public */,
       5,    0,   88,    2, 0x0a /* Public */,
       6,    0,   89,    2, 0x0a /* Public */,
       7,    1,   90,    2, 0x0a /* Public */,
       9,    1,   93,    2, 0x0a /* Public */,
      11,    1,   96,    2, 0x08 /* Private */,
      14,    0,   99,    2, 0x08 /* Private */,
      15,    0,  100,    2, 0x08 /* Private */,
      16,    1,  101,    2, 0x08 /* Private */,
      18,    0,  104,    2, 0x08 /* Private */,
      19,    0,  105,    2, 0x08 /* Private */,
      20,    0,  106,    2, 0x08 /* Private */,
      21,    0,  107,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::LongLong,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QGstreamerPlayerControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QGstreamerPlayerControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setPosition((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 1: _t->play(); break;
        case 2: _t->pause(); break;
        case 3: _t->stop(); break;
        case 4: _t->setVolume((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->setMuted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->updateSessionState((*reinterpret_cast< QMediaPlayer::State(*)>(_a[1]))); break;
        case 7: _t->updateMediaStatus(); break;
        case 8: _t->processEOS(); break;
        case 9: _t->setBufferProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->handleInvalidMedia(); break;
        case 11: _t->handleResourcesGranted(); break;
        case 12: _t->handleResourcesLost(); break;
        case 13: _t->handleResourcesDenied(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QMediaPlayer::State >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QGstreamerPlayerControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QMediaPlayerControl::staticMetaObject>(),
    qt_meta_stringdata_QGstreamerPlayerControl.data,
    qt_meta_data_QGstreamerPlayerControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QGstreamerPlayerControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGstreamerPlayerControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QGstreamerPlayerControl.stringdata0))
        return static_cast<void*>(this);
    return QMediaPlayerControl::qt_metacast(_clname);
}

int QGstreamerPlayerControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMediaPlayerControl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
